function bit_band = analyze_slice(x, fs, b, do_plot)
%ANALYZE_SLICE Analyze a slice of a sound sample to compute bit allocation
%   
%   The function performs the analysis of a slice of a sound sample
%   following the Psychoacoustic Model 1 of the MPEG standard. The model
%   only takes frequency domain masking into account. 
%
%   The analysis is performed in the following steps:
%    1. The power spectrum of the sample is computed using a Hann window.
%    2. Tonal and noise maskers are identified.
%    3. Masking thresholds are computed from the masker components.
%    4. The number of quantization bits are determined based on the global
%       masking threshold.
%
%   BIT_BAND = ANALYIZE_SLICE(X, FS, B, DO_PLOT) Analyzes the sound sample
%       slice X (a vector of samples) having the sampling frequency FS (a 
%       scalar value), and bit depth B (a positive integer). The output
%       BIT_BAND is the number representation bits needed in each critical 
%       band. The auxiliary input variable DO_PLOT is an integer value, 
%       which can be set as follows:
%           0: do not plot anything
%           1: plot masking curves and global masking threshold
%           2: plot noise to mask ratio and bit allocation
%
%   Example:
%       f = 1500;           % Frequency [Hz]
%       fs = 44100;         % Sampling frequncy [Hz]
%       N = 512;            % Window size
%       b = 16;             % Sample bit size
%       t = (0 : N-1).'/fs; % Time scale
%       % Generate samples
%       x = 2^(b-1) * 0.5 * sin(2*pi*f*t);
%       % Perform analysis
%       analyze_slice(x, fs, b, 1);
%
%   See also: critical_bands, global_masking_thresholds, hearing_threshold,
%       maskers, masking_thresholds, reorganize_maskers,

if nargin < 4
    do_plot = 0;
end

if nargin < 3 || isempty(b)
    b = 16;
end

if ~isvector(x) || ~isreal(x)
    error('analyze_slice:invalid_argument', ...
        'Input parameter ''X'' must be a vector of real numbers');
end

% Check if size too big
if length(x) > 4096
    error('analyze_slice:invalid_size', ...
        'Maximum size of slice ''X'' is 4096, length of input is: %d', length(x));
end

N_fft = length(x);
f = (0 : N_fft/2).'/N_fft*fs;
x_norm = x(:) / (N_fft * 2^(b-1));

% Analysis window
win = hann(N_fft);
% Equivalent noise bandwidth of the window
enbw = sum(win) / N_fft;

% Nominal dBSPL corresponding to full scale
PN = 90.3;

% Step (1): Calculate power spectrum
P = PN + 20*log10(abs(fft(win.*x_norm))/enbw);
P = [P(1); P(2 : N_fft/2); P(N_fft/2+1)];

% Step (2a): Find tonal and noise maskers
[P_TM, P_NM] = maskers(P, fs);
% Step (2b): Reorganize maskers
[P_mask, mask_sel] = reorganize_maskers(P_TM, P_NM, fs);

% Step (3a): Compute individual masking thresholds
[T_TM, T_NM] = masking_thresholds(P_mask, mask_sel, fs);
% Step (3b): Compute global masking thresholds
Tq = hearing_threshold(f);
TG = global_masking_thresholds(T_TM, T_NM, Tq, mask_sel);

% Step (4): Compute bit allocation
[bit_gain, TG_crit] = bit_allocation(f, TG, b);
bit_band = b - bit_gain;

% Plot as needed
switch do_plot
    case 1
        plot_maskers(fs, P, Tq, P_mask, mask_sel, TG, T_TM, T_NM);
    case 2
        plot_nmr(f, TG, TG_crit);
    otherwise
        % No plot
end

end % of function analyze_slice

