function [bit_bands, Tg_crit] = bit_allocation(f, Tg, b)
% BIT_ALLOCATION Compute bit gain in critical bands from masking threshold
%
%   Bit allocation is computed based on the noise to mask ratio, which is
%   the difference of the global masking curve and the nominal noise level 
%   of the sample. In this routine a nominal noise level of 0 dBSPL is
%   assumed for a 16-bit recording. If the number of quantization bits is 
%   reduced by B, then the noise in that band is increased by 6*B dB. This 
%   is acceptible if the masking threshold is above of the increased noise
%   level. Therefore, the bit gain in the i-th band is found as:
% 
%   $$b(i) = \lfloor \frac{T_{min}(i) - N_{nom}}{6} \rfloor$$
%
%   B = BIT_ALLOCATION(F, TG, B) Computes the bit gain in critical bands 
%       from the global masking threshold TG. The masking threshold is 
%       given in dBSPL units over the frequency scale F given in Hertz 
%       units. The input vectors F and TG must have the same size, the
%       input parameter B is the bit depth of the recording. If B is not 
%       set a default value of B = 16 is assumed.
%
%   See also: global_masking_thresholds

if nargin < 3
    b = 16;
end

[~, N_bands] = critical_bands();
band_idx = critical_band_index(f);

% Assemble global threshold for each critical band
Tg_crit = zeros(N_bands, 1);

% Compute threshold in each critical band
for j = 1 : N_bands
    % Select frequency bins in the current band
    idx = (band_idx == j);
    % The global threshold in the given band is the minimal that of bins
    if any(idx)
        Tg_crit(j)= min(Tg(idx));
    else
        Tg_crit(j) = 0;
    end
end

% Compute noise to mask ratio (NMR) assuming a noise level of 0 dBSPL for a
% 16-bit recording. If the number of bits is greater, a lower noise level
% is assumed.
NMR = Tg_crit - (-6*(b-16));
% Number of bits to gain is proportional to the NMR 
bit_bands = max(min(floor(NMR/6), b), 0);

end % of function bit_allocation