function [P_TM, P_NM, ST] = maskers(P, fs)
%MASKERS Compute tonal and noise maskers from a dB power spectrum
%   
%   [P_TM, P_NM, ST] = MASKERS(P, FS) Computes the tonal and noise maskers
%       from a power spectrum P. P_TM and P_NM is the resulting power of 
%       tonal and noise maskers, respectively. ST is the tonal set. The
%       input parameter FS defines the sampling frequency which must be
%       given in Hz units. The one-sided power spectrum P must be given in
%       dB units.
%
%   See also: tonal_set

N_fft = (length(P)-1)*2;

% Compute tonal set
[ST, cmp1, cmp2] = tonal_set(P);
fST = find(ST);

% (1) Assemble tonal maskers
P_TM = nan(size(P, 1), 1);
% Fill tonal masker values according to eq. (5.23)
% This means a sum over the tonal spectral components and their neighbors
P_TM(ST) = 10*log10(sum(10.^(0.1*[P(fST+1), P(fST),  P(fST-1)]), 2));

% (2) Assemble noise maskers
P_NM = nan(N_fft/2+1, 1);
% Full comparison matrix
cmp = cmp1 | cmp2 | logical(speye(N_fft/2+1));

% Find critical band indices
f = (0 : N_fft/2).' / N_fft * fs;
band_idx = critical_band_index(f);

[~, N_bands] = critical_bands();
f_mid = zeros(N_bands, 1);

f = (0 : N_fft/2+1)/N_fft*fs;

% Traverse critical bands
for i_band = 1 : N_bands
    f_sel = (band_idx == i_band);
    f_sel(1) = false;   % Skip DC
    % Geometric mean of frequency lines
    f_mid(i_band) = prod(f(f_sel))^(1 / sum(double(f_sel)));
    % Find the closest frequency bin
    [~, bin_idx] = min(abs(f_mid(i_band) - f));
    % Frequency lines to be excluded from summing
    f_exc = any(cmp(ST, :), 1);
    f_sum = f_sel & ~f_exc.';
    % Sum the power
    if (any(f_sum))
        P_NM(bin_idx) = 10*log10(sum(10.^(0.1 * P(f_sum))));
    end
end % of traversing critical bands

end % of function maskers
