function [P_mask, mask_sel] = reorganize_maskers(P_TM, P_NM, fs)
%REORGANIZE_MASKERS Reorganize masking components
%   
%   Masker reorganization performs a filtering of masking components in
%   order to reduce the number of maskers for the subsequent steps of audio
%   coding. Maskers below the hearing threshold are disregarded, other
%   maskers close to each other in frequency are unified.
%
%   [P_MASK, MASK_SEL] = REORGANIZE_MASKERS(P_TM, P_NM, FS) Reorganizes the
%       tonal maskers P_TM and noise maskers P_NM. FS is the sampling
%       frequency of the input signal.
%       
%       Output P_MASK will contain the power of the reorganized masking
%       components. MASK_SEL tells if a selected masking component is a
%       tonal masker (value: 1), noise masker (value: 2), or there is no
%       masker at the given bin (value: 0).
%
%   See also: hearing_threshold, hz2bark, maskers, masking_thresholds

N_fft = (length(P_TM)-1)*2;

% The frequency scale in Hz and Bark units
f = (0 : N_fft/2).'/N_fft * fs;
b = hz2bark(f);

[P, mask_sel] = max([P_TM P_NM], [], 2);
% Compute the hearing threshold
Tq = hearing_threshold(f);

% Disregard maskers below the hearing threshold
sel = P < Tq;
P(sel) = NaN;
mask_sel(sel) = 0;

P_mask = P;

% Sliding window with a width of 1 bark
for i = 1 : N_fft/2+1
    if isnan(P(i))
        continue
    end
    d_bark = b(i) - b;
    % Find the FFT bin indexes that are within 
    cmp_bins = (abs(d_bark) <= 0.5);
    if (any(P(cmp_bins) > P(i)))
        P_mask(i) = nan;
        mask_sel(i) = 0;
    end
end

end % of function reorganize_maskers
