function [x_out, bit_bins] = requantize_slice(x_in, fs, b, bit_bands, method)
%REQUANTIZE_SLICE Requantize a slice of an audio sample
%   
%   This routine performs the requantization of a slice of an audio sample
%   in the frequency domain.
%
%   [X_OUT, BIT_BINS] = REQUANTIZE_SLICE(X_IN, FS, B, BIT_BANDS, METHOD)
%       Performs the requantization of the slice given in the vector X_IN
%       with the sampling frequncy FS (given in Hertz units) and original
%       bit depth B. The bit numbers for resampling are given in the vector
%       BIT_BANDS which should have the same size as the number of critical
%       bands. To obtain BIT_BANDS one can utilize the function 
%       ANALAYZE_SLICE. The variable METHOD specifies the frequency 
%       transformation method which can be one of the following:
%           'dct' - Discrete Cosine Transform (Default)
%           'fft' - Fast Fourier Transform (requantize both real and imag)
%           'fftabs' - FFT but requantize amplitude only
%
%       The output X_OUT is the resampled slice transformed back into the 
%       time domain. The optional output BIT_BINS gives the number of bits 
%       required for storing the samples of the window in the frequency 
%       domain.
%
%   Example:
%       fs = 44100;                             % Sampling frequency [Hz]
%       N = 512;                                % Number of samples
%       t = (0 : N-1).'/fs;                     % Time vector
%       f0 = 1300;                              % Signal frequency [Hz]
%       b = 16;                                 % Bit depth
%       A = 0.6;                                % Signal amplitude
%       x = 2^(b-1)*A*sin(2*pi*f0*t);           % The signal
%       bit_band = analyze_slice(x, fs, b);     % Analysis
%       method = 'dct';                         % Transformation method
%       % Perform requantization 
%       y = requantize_slice(x, fs, b, bit_band, method);
%       plot(t, [x, y, x-y]);                   % Plot samples and diff.
%
%   See also: analyze_slice, requantize


% Check input arguments
if nargin < 5
    method = 'dct';
else
    if ~ischar(method)
        error('requantize_slice:invalid_argument', ...
            'Input parameter ''METHOD'' must be a string');
    end
end

x_in = x_in(:);
N_bins = length(x_in);

switch lower(method)
    case 'dct'
        f = (0 : N_bins-1)/N_bins*fs/2;
    case {'fft', 'fftabs'}
        f = (0 : N_bins-1)/N_bins*fs;
        f = f(1 : ceil((length(f)+1)/2));
    otherwise
        error('requantize_slice:unknown_method', ...
            'Unknown method: %s', method);
end

bit_bins = band2freq(bit_bands, f);

switch lower(method)
    case 'dct'
        scale = sqrt([1; 2*ones(N_bins-1,1)]/N_bins); 
        X = dct(x_in) .* scale;
        X_re = requantize(X, b, bit_bins);
        bit_bins(X_re == 0) = 1;
        x_out = idct(X_re) ./ scale;
    case 'fft'
        scale = [1; 2*ones(floor(N_bins/2), 1)]/N_bins;
        scale = [scale; flipud(scale(2:end-1))];
        X = fft(x_in) .* scale;
        % Make one-sided
        X = X(1 : floor(N_bins/2)+1);
        % Requantize real and imaginary parts separately
        X_re_r = requantize(real(X), b, bit_bins);
        X_re_i = requantize(imag(X), b, bit_bins);
        bit_bins = 2*bit_bins;
        bit_bins(X_re_r == 0 & X_re_i == 0) = 1;
        x_out = ifft(X_re_r + 1i*X_re_i, N_bins, 'symmetric') ./ scale;
    case 'fftabs'
        scale = [1; 2*ones(floor(N_bins/2), 1)]/N_bins;
        scale = [scale; flipud(scale(2:end-1))];
        X = fft(x_in) .* scale;
        % Make one-sided
        X = X(1 : floor(N_bins/2)+1);
        % Requantize real and imaginary parts separately
        X_abs = abs(X);
        X_phase = angle(X);
        % Requantize absolute value only
        X_re_abs = requantize(X_abs, b, bit_bins);
        x_out = ifft(X_re_abs .* exp(1i*X_phase), N_bins, 'symmetric') ./ scale;
end

end % of function requantize_slice
