function [x_out, bit_out, t, f] = requantize_windows(x_in, fs, b, method, N_win, N_over, winfun)
% REQUANTIZE_WINDOWS Requantize a sound sample as a sequence of windows
%
%   [X_OUT, BITW, T, F] = REQUANTIZE_WINDOWS(X,FS,B,METHOD,N_WIN,N_OL,WFN)
%       Performs the requantization of the sound sample given in the column
%       vector X with sampling frequency FS (in Hertz) and bit depth B. 
%       The frequency transformation method is set by the parameter METHOD.
%       METHOD must be a string and can have one of the following values:
%           'dct' - Discrete Cosine Transform (recommended)
%           'fft' - Fast Fourier Transform (requantize both real and imag)
%           'fftabs' - FFT but requantize amplitude only
%       The number of samples in each window is N_WIN and the number of
%       overlapping samples between the windows is N_OL. The parameter WFN
%       should be a function handle that selects the window to be used. If 
%       WFN is not given, WFN = @hann is used as default. The window 
%       function should be symmetric, such that the sum of subsequent 
%       overlapping windows gives unity.
%
%       Recommended values for requantizing audio samples:
%           N_WIN = 128, 256, 512, 1024, or 2048
%           N_OL  = 1/8 .. 1/2 of N_WIN
%
%       The output sample X_OUT is the requantized sample transformed back
%       into the time domain. BITW is a matrix containing the resulting
%       number of bits for each time window and each frequency bin defined
%       by the selected frequency transformation method. The i-th row of 
%       the matrix BITW holds the bit numbers for the i-th analysis window.
%       The output vectors T and F are the time and frequency scales for
%       plotting the data in BITW.
%
%   Example:
%       [x, fs] = audioread('example.wav');     % Load audio file
%       b = 16;                                 % Simulate 16-bit depth
%       x = x(:, 1)*2^(b-1);                    % Up-scale 1 channel only
%       N_win = 512;                            % Samples in one window
%       N_ol = 128;                             % Overlapping of windows
%       method = 'dct';                         % Transformation method
%       % Perform requantization of the whole sample
%       [y, bo, t, f] = requantize_windows(x, fs, b, method, N_win, N_ol);
%       subplot(2, 1, 1);
%       plot([x, y]);           % Plot original and requantized samples
%       subplot(2, 1, 2);
%       pcolor(f, t, bo);       % Show used number of bits in each window
%       shading interp;         % Hide black lines in color plot
%       colorbar;               % Show the colorbar legend
%       sound(y / 2^(b-1), fs); % Play the requantized sample
%
%   See also: analyze_slice, requantize_slice

% Check input
if nargin < 7
    winfun = @hann;
elseif ~isa(winfun, 'function_handle')
    error('reqauntize_windows:invalid_argument', ...
        'Input parameter WFN must be a function handle');
end

N_samp = length(x_in);
N_slices = floor((N_samp - N_win) / (N_win - N_over)) + 1;

x_out = zeros(N_samp, 1);

win_norm = ones(N_win, 1);
win_start = ones(N_win, 1);
win_stop = ones(N_win, 1);
ww = winfun(N_over * 2);

% Assemble normal, start and stop windows
win_norm(1 : N_over) = ww(1 : N_over);
win_stop(1 : N_over) = ww(1 : N_over);

win_norm(end - N_over + 1 : end) = ww(N_over + 1 : end);
win_start(end - N_over + 1 : end) = ww(N_over + 1 : end);

% Go through all slices
for i_slice = 1 : N_slices
    progbar(1, N_slices, i_slice);
    idx = (i_slice - 1) * (N_win - N_over) + (1 : N_win);
    % Select the slice
    x_slice = x_in(idx);
    % Analyze to compute bit allocation
    bit_bands = analyze_slice(x_slice, fs, b, false);
    % Requantize the slice using the selected method
    [x_re_slice, bit_win] = requantize_slice(x_slice, fs, b, bit_bands, method);
    
    if (i_slice == 1)
        x_out(idx) = x_out(idx) + x_re_slice .* win_start;
    elseif (i_slice == N_slices)
        x_out(idx) = x_out(idx) + x_re_slice .* win_stop;
    else
        x_out(idx) = x_out(idx) + x_re_slice .* win_norm;
    end
    
    if (i_slice == 1)
        bit_out = zeros(N_slices, length(bit_win));
    end
    bit_out(i_slice, :) = bit_win;
end % of cycle through slices

% Time vector
t = (0 : N_slices-1).' * (N_win - N_over) / fs;
% Frequency vector
switch method
    case 'dct'
        f = (0 : N_win-1) / N_win * fs / 2;
    case {'fft', 'fftabs'}
        f = (0 : N_win/2) / N_win * fs;
end

end % of function requantize_windows
