function [ST, cmp1, cmp2] = tonal_set(P)
%TONAL_SET Find tonal set in a power spectrum
%   
%   The tonal set of a power spectrum is formed by the peaks of the
%   spectrum, i.e., components where the spectrum is stronger than at the
%   neighboring bins.
%
%   ST = TONAL_SET(P) Gives the tonal set ST as a logical vector based on
%       the one-sided power spectrum P. The i-th element of ST is true if 
%       the i-th element of P is a tonal component and false otherwise. 
%       P should be a vector and given in dB.
%
%   [ST, CMP1, CMP2] = TONAL_SET(P) also gives the comparison matrices CMP1
%       and CMP2 (logical square sparse matrices) for later usage.
%
%   See also: cmp_matrices

N_fft = (length(P)-1)*2;
[cmp1, cmp2] = cmp_matrices(N_fft);

PmP = bsxfun(@minus, P, P.');
X = sparse(N_fft/2+1, N_fft/2+1);
X(cmp1) = PmP(cmp1) > 0;
Y = sparse(N_fft/2+1, N_fft/2+1);
Y(cmp2) = PmP(cmp2) > 7;

% Assemble tonal set
ST = sum(X, 2) == sum(cmp1, 2) & sum(Y, 2) == sum(cmp2,2); 
ST(1) = false;
ST(end) = false;

end % of function tonal_set
