function [T_TM, T_NM, SF] = masking_thresholds(P_mask, mask_sel, fs)
% MASKING_THRESHOLDS Masking thresholds from tonal and noise maskers
%
%   Masking thresholds are computed by making use of the spreading function
%   of the basilar membrane. Tonal and noise masking components have
%   significantly different spreading, which is exploited in this fucntion.
%  
%   [T_TM, T_NM, SF] = MASKING_THRESHOLDS(P_MASK, MASK_SEL, FS) Computes
%       the tonal and noise masking thresholds T_TM and T_NM using the
%       masking components defined by P_MASK and MASK_SEL. The inputs
%       P_MASK and MASK_SEL can be computed by using the function
%       REORGANIZE_MASKERS. FS is the sampling frequency of the signal.
%       The optional output SF is the spreading function containing the
%       estimated response of the basilar membrane to a narrowband
%       excitation having unit amplitude.
%
%   See also: hz2bark, maskers, reorganize_maskers

N_fft = (length(P_mask)-1)*2;
f = (0 : N_fft/2).' / N_fft * fs;
SF = nan(N_fft/2+1, N_fft/2+1);
z = hz2bark(f);

P_TM = zeros(N_fft/2+1, 1);
P_NM = zeros(N_fft/2+1, 1);

P_TM(mask_sel==1) = P_mask(mask_sel==1);
P_NM(mask_sel==2) = P_mask(mask_sel==2);

% Compute bark difference (dz(i,j) = z_i - z_j)
dz = bsxfun(@minus, z, z.');

% Far lower components: bark difference between -3 and -1
val1 = bsxfun(@minus, 17*dz, 0.4*P_TM(:).') + 11;
sel1 = dz >= -3 & dz <= -1;
SF(sel1) = val1(sel1);

% Lower components: bark difference between -1 and 0
val2 = bsxfun(@times, dz, 0.4*P_TM(:).' + 6);
sel2 = dz > -1 & dz < 0;
SF(sel2) = val2(sel2);

% Higher components: bark difference between 0 and 1
sel3 = dz >= 0 & dz < 1;
SF(sel3) = -17 * dz(sel3);

% Far higher components: bark difference between 1 and 8
sel4 = dz > 1 & dz <= 8;
val4 = bsxfun(@minus, bsxfun(@times, dz, 0.15*P_TM(:).' -17), 0.15*P_TM(:).');
SF(sel4) = val4(sel4);

T_TM = bsxfun(@plus, SF, P_TM(:).' - 0.275*z(:).' - 6.025);
T_NM = bsxfun(@plus, SF, P_NM(:).' - 0.175*z(:).' - 2.025);

end % of function masking_thresholds

